==> orange_dam-1.5.1.tar.gz: orange_dam/src/OrangeDamQueueDataManager.php <==
<?php

namespace Drupal\orange_dam;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\State\State;

/**
 * Manages retrieving a list of updated content and building a queue.
 */
class OrangeDamQueueDataManager implements OrangeDamQueueDataManagerInterface {

  /**
   * The log channel to use.
   */
  public const LOGGER_CHANNEL = 'orange_dam.queue';

  /**
   * The state key that tracks the last time we checked for updates to content.
   */
  public const UPDATED_ITEM_STATE_TIME_KEY = 'orange_dam_item_update_last_run_time';

  /**
   * The state key that tracks the last time we checked for updates to keywords.
   */
  public const UPDATED_KEYWORD_STATE_TIME_KEY = 'orange_dam_keyword_update_last_run_time';

  /**
   * The state key that tracks the last time we checked for deleted content.
   */
  public const DELETED_CONTENT_STATE_TIME_KEY = 'orange_dam_deleted_content_last_run_time';

  /**
   * The state key that tracks the last time we checked for deleted keywords.
   */
  public const DELETED_KEYWORD_STATE_TIME_KEY = 'orange_dam_deleted_keyword_last_run_time';

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The Orange DAM configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructor.
   */
  public function __construct(
    protected LoggerChannelFactoryInterface $loggerChannelFactory,
    protected QueueFactory $queueFactory,
    protected OrangeDamApi $orangeDamApi,
    protected State $state,
    protected OrangeDamMigrationDataManager $migrationDataManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected OrangeDamConfigurationManager $orangeDamConfiguration,
  ) {
    $this->logger = $loggerChannelFactory->get(static::LOGGER_CHANNEL);
  }

  /**
   * Queue all updated keywords and content.
   */
  public function syncUpdates() {
    $this->queueKeywordDeletions();
    $this->queueContentDeletions();
    $this->queueUpdatedKeywords();
    $this->queueUpdatedItems();
  }

  /**
   * The log for queue related issues.
   *
   * @return \Drupal\Core\Logger\LoggerChannelInterface
   *   The logger for this service.
   */
  public function logger() {
    return $this->logger;
  }

  /**
   * {@inheritdoc}
   */
  public function queueUpdatedItems(DrupalDateTime $timeUpdatedSince = NULL) {
    // Use the passed time if available. Fallback to the updated time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeUpdatedSince)) {
      $timeUpdatedSince = new DrupalDateTime($this->state->get(static::UPDATED_ITEM_STATE_TIME_KEY, 'now'));
    }
    $time = $timeUpdatedSince->setTimezone(new \DateTimeZone('utc'))->format('c');
    $this->logger->notice(
      'Queueing all Orange DAM content for re-syncing with changes since :time.', [
        ':time' => $time,
      ]);
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Get the items deleted since the given time until now.
    $itemCount = $this->orangeDamApi->getContent($timeUpdatedSince, $timeNow);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the last time this was run for future checks.
    $this->state->set(static::UPDATED_ITEM_STATE_TIME_KEY, $timeNow->format('c'));
    // Log the results.
    $this->logger->notice(
      'Queued :count Orange DAM content item(s) for re-syncing with changes after :time.', [
        ':time' => $time,
        ':count' => $itemCount,
      ]);
    return $itemCount;
  }

  /**
   * {@inheritdoc}
   */
  public function queueUpdatedKeywords(DrupalDateTime $timeUpdatedSince = NULL) {
    // Use the passed time if available. Fallback to the updated time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeUpdatedSince)) {
      $timeUpdatedSince = new DrupalDateTime($this->state->get(static::UPDATED_KEYWORD_STATE_TIME_KEY, 'now'));
    }
    // Standardize the timezone.
    $timeUpdatedSince->setTimezone(new \DateTimeZone('utc'));
    // Record the time of the request.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Log the action being taken.
    $this->logger->notice(
      'Queueing all Orange DAM keywords for re-syncing that have been updated since :time.', [
        ':time' => $timeUpdatedSince->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $updatedKeywordCount = $this->orangeDamApi->getKeywords($timeUpdatedSince, $timeNow);
    if ($updatedKeywordCount === FALSE) {
      return FALSE;
    }
    // Record the last time this was run for future checks.
    $this->state->set(static::UPDATED_KEYWORD_STATE_TIME_KEY, $timeNow->format('c'));
    // Log the results.
    $this->logger->notice(
      'Queued :count Orange DAM keyword(s) for re-syncing.', [
        ':count' => $updatedKeywordCount,
      ]);
    return $updatedKeywordCount;
  }

  /**
   * Queue updates for all items of a given keyword type.
   *
   * @param string[] $keywordTypes
   *   The keyword types to queue updates for.
   *
   * @return int
   *   The number of queued keywords.
   */
  public function queueKeywordByType(array $keywordTypes): int {
    // Validate the keyword types.
    foreach ($keywordTypes as $keywordType) {
      if (!$this->validateKeywordType($keywordType)) {
        $this->logger->error('Invalid keyword type :type requested for syncing.', [
          ':type' => $keywordType,
        ]);
      }
    }
    // Request the data from the API and queue the results.
    $updatedKeywordCount = $this->orangeDamApi->getKeywords(
      NULL,
      NULL,
      $keywordTypes,
    );
    // Log the results.
    $this->logger->notice(
      'Queueing :count Orange DAM :type keywords(s) for re-syncing.', [
        ':count' => $updatedKeywordCount,
        ':type' => implode(', ', $keywordTypes),
      ]);
    return $updatedKeywordCount;
  }

  /**
   * {@inheritdoc}
   */
  public function updateSingleItem(string $systemIdentifier) {
    if ($item = $this->orangeDamApi->getContentItem($systemIdentifier)) {
      $this->logger->notice(
        'Orange DAM item :system_identifier has been successfully queued for updating in the :queue queue.', [
          ':system_identifier' => $systemIdentifier,
          ':queue' => $this->migrationDataManager->getItemQueueName($item),
        ]);
      return $item;
    }
    $this->logger->error(
      'Failed to retrieve Orange DAM item :system_id from the Orange DAM API.', [
        ':system_id' => $systemIdentifier,
      ]
    );
    return FALSE;
  }

  /**
   * Queue updates for all items of a given content type.
   *
   * @param array $contentTypes
   *   The content types to queue updates for.
   * @param int $limit
   *   The maximum number of items, rounded to the nearest page size.
   * @param int $pageNumber
   *   The optional initial page number for paged requests.
   */
  public function queueContentByType(array $contentTypes, int $limit = 0, int $pageNumber = OrangeDamApi::INITIAL_PAGE_NUMBER) {
    $itemCount = 0;
    // Validate the content types.
    foreach ($contentTypes as $contentType) {
      if (!$this->validateContentType($contentType)) {
        $this->logger->error(
          'Invalid content type :type requested for syncing.', [
            ':type' => $contentType,
          ]);
        continue;
      }
      // Request the data from the API and queue the results.
      $itemCount += $this->orangeDamApi->getContent(
        NULL,
        NULL,
        [$contentType],
        $limit,
        $pageNumber,
      );
    }
    // Record the total number of items queued.
    $this->logger->notice(
      'Queued :count Orange DAM :types item(s) for re-syncing.', [
        ':count' => $itemCount,
        ':types' => implode(', ', $contentTypes),
      ]);
  }

  /**
   * {@inheritdoc}
   */
  public function queueContentDeletions(DrupalDateTime $timeDeletedSince = NULL) {
    // Use the passed time if available. Fallback to the deleted time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeDeletedSince)) {
      $timeDeletedSince = new DrupalDateTime($this->state->get(static::DELETED_CONTENT_STATE_TIME_KEY, 'now'));
    }
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Log the action being taken.
    $this->logger->notice(
      'Syncing deletions for all Orange DAM content that has been deleted since :time.', [
        ':time' => $timeDeletedSince->setTimezone(new \DateTimeZone('utc'))->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $itemCount = $this->orangeDamApi->getDeletedContent($timeDeletedSince, $timeNow);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the time since the request succeeded.
    $this->state->set(static::DELETED_CONTENT_STATE_TIME_KEY, $timeNow->format('c'));
    // Display a final summary.
    $this->logger->notice(
      'Queued deletions for :count Orange DAM content items.', [
        ':count' => $itemCount,
      ]);
    return $itemCount;
  }

  /**
   * {@inheritdoc}
   */
  public function queueKeywordDeletions(DrupalDateTime $timeDeletedSince = NULL) {
    // Use the passed time if available. Fallback to the deleted time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeDeletedSince)) {
      $timeDeletedSince = new DrupalDateTime($this->state->get(static::DELETED_KEYWORD_STATE_TIME_KEY, 'now'));
    }
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));

    $this->logger->notice(
      'Syncing deletions for all Orange DAM keywords that have been deleted since :time.', [
        ':time' => $timeDeletedSince->setTimezone(new \DateTimeZone('utc'))->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $itemCount = $this->orangeDamApi->getDeletedKeywords($timeDeletedSince);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the time since the request succeeded.
    $this->state->set(static::DELETED_KEYWORD_STATE_TIME_KEY, $timeNow->format('c'));
    return $itemCount;
  }

  /**
   * Deletes an Orange DAM item during queue processing.
   *
   * @param \Drupal\orange_dam\OrangeDamItemInterface $item
   *   The Orange DAM item to use as the source of the deletion operation.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *
   * @see \Drupal\orange_dam\Plugin\QueueWorker\OrangeDamDeletionQueueWorker
   */
  public function processDeletedQueueItem(OrangeDamItemInterface $item) {
    // Attempt to find the item in a migration.
    if (!$migration = $this->migrationDataManager->getMigrationBySourceId($item->id())) {
      $this->logger->error('Failed to delete an item of type ":type" while processing the deletion queue. Unable to find a migration with ":id" as a source identifier.', [
        ':id' => $item->id(),
        ':type' => $item->type(),
      ]);
      return;
    }
    // Get the destination entity ID and type values.
    $entityIds = $migration->getIdMap()->lookupDestinationIds([$item->id()]);
    // The result of the destination ID lookup above returns a nested array of
    // source keys and their 1 to many destination keys even when there are no
    // results, so we filter the results to simplify the empty check below.
    $entityIds = array_filter(array_pop($entityIds));
    $entityType = $migration->getDestinationPlugin()->getDerivativeId();
    // Validate that a destination entity was found.
    if ($entityIds === []) {
      $this->logger->error('Failed to delete an item of type ":type" while processing the deletion queue. Unable to find a migration destination item with ":id" as a source identifier.', [
        ':id' => $item->id(),
        ':type' => $item->type(),
      ]);
      return;
    }
    // Load the entities.
    $entities = $this->entityTypeManager
      ->getStorage($entityType)
      ->loadMultiple($entityIds);
    // Validate that entities were found.
    if ($entities === []) {
      $this->logger->error('Failed to find an entity with the ID :id of the Orange Logic type ":type" while processing deletions for item ":source_id".', [
        ':id' => implode(', ', $entityIds),
        ':type' => $item->type(),
        ':source_id' => $item->id(),
      ]);
      return;
    }
    // Delete the entities.
    foreach ($entities as $entity) {
      try {
        $entity->delete();
        $this->logger->info('Deleted entity (:type::id) ":title" during Orange DAM deletion syncing.', [
          ':type' => $entity->getEntityType()->id(),
          ':id' => $entity->id(),
          ':title' => $entity->label(),
        ]);
      }
      catch (EntityStorageException) {
        $this->logger->error('Failed to delete entity (:type::id) ":title" during Orange DAM deletion syncing.', [
          ':type' => $entity->getEntityType(),
          ':id' => $entity->id(),
          ':title' => $entity->label(),
        ]);
        continue;
      }
    }
  }

  /**
   * Validate if a content type is valid.
   *
   * @param string $contentType
   *   The content type name.
   *
   * @return bool
   *   Indication if the content type is valid.
   */
  public function validateContentType($contentType): bool {
    return in_array($contentType, $this->orangeDamConfiguration->getContentTypeNames());
  }

  /**
   * Validate if a keyword type is valid.
   *
   * @param string $keywordType
   *   The keyword type name.
   *
   * @return bool
   *   Indication if the keyword type is valid.
   */
  public function validateKeywordType($keywordType): bool {
    return in_array($keywordType, $this->orangeDamConfiguration->getKeywordTypes());
  }

  /**
   * Generate the URL to a file asset for a given item.
   *
   * The content type is optional, but if it is omitted it adds a second API
   * call to this process.
   *
   * @param string $systemId
   *   The item's System ID.
   * @param string $contentType
   *   The item's content type.
   *
   * @return string|false
   *   The asset URL if one was generated.
   */
  public function getAssetUrl(string $systemId, string $contentType = NULL) {
    // Get the content type of item if it was not passed in.
    if (is_null($contentType)) {
      $item = $this->orangeDamApi->getContentItem($systemId);
      $contentType = $item ? $item->type() : NULL;
    }
    // Confirm that a content type was found.
    if (!$contentType) {
      $this->logger->error(
        'Unable to determine the content type of the item with system ID :system_id for asset URL generation.', [
          ':system_id' => $systemId,
        ]);
      return FALSE;
    }
    // Get the asset format and verify that a value was found.
    $assetFormat = $this->orangeDamConfiguration->getAssetFormat($contentType);
    if (!$assetFormat) {
      return FALSE;
    }
    // Generate the asset URL.
    return $this->orangeDamApi->getAssetUrl(
      $systemId,
      $assetFormat,
    );
  }

}

==> orange_dam-1.6.0.tar.gz: orange_dam/src/OrangeDamQueueDataManager.php <==
<?php

namespace Drupal\orange_dam;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\State\State;

/**
 * Manages retrieving a list of updated content and building a queue.
 */
class OrangeDamQueueDataManager implements OrangeDamQueueDataManagerInterface {

  /**
   * The log channel to use.
   */
  public const LOGGER_CHANNEL = 'orange_dam.queue';

  /**
   * The state key that tracks the last time we checked for updates to content.
   */
  public const UPDATED_ITEM_STATE_TIME_KEY = 'orange_dam_item_update_last_run_time';

  /**
   * The state key that tracks the last time we checked for updates to keywords.
   */
  public const UPDATED_KEYWORD_STATE_TIME_KEY = 'orange_dam_keyword_update_last_run_time';

  /**
   * The state key that tracks the last time we checked for deleted content.
   */
  public const DELETED_CONTENT_STATE_TIME_KEY = 'orange_dam_deleted_content_last_run_time';

  /**
   * The state key that tracks the last time we checked for deleted keywords.
   */
  public const DELETED_KEYWORD_STATE_TIME_KEY = 'orange_dam_deleted_keyword_last_run_time';

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The Orange DAM configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructor.
   */
  public function __construct(
    protected LoggerChannelFactoryInterface $loggerChannelFactory,
    protected QueueFactory $queueFactory,
    protected OrangeDamApi $orangeDamApi,
    protected State $state,
    protected OrangeDamMigrationDataManager $migrationDataManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected OrangeDamConfigurationManager $orangeDamConfiguration,
  ) {
    $this->logger = $loggerChannelFactory->get(static::LOGGER_CHANNEL);
  }

  /**
   * Queue all updated keywords and content.
   */
  public function syncUpdates() {
    $this->queueKeywordDeletions();
    $this->queueContentDeletions();
    $this->queueUpdatedKeywords();
    $this->queueUpdatedItems();
  }

  /**
   * The log for queue related issues.
   *
   * @return \Drupal\Core\Logger\LoggerChannelInterface
   *   The logger for this service.
   */
  public function logger() {
    return $this->logger;
  }

  /**
   * {@inheritdoc}
   */
  public function queueUpdatedItems(DrupalDateTime $timeUpdatedSince = NULL) {
    // Use the passed time if available. Fallback to the updated time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeUpdatedSince)) {
      $timeUpdatedSince = new DrupalDateTime($this->state->get(static::UPDATED_ITEM_STATE_TIME_KEY, 'now'));
    }
    $time = $timeUpdatedSince->setTimezone(new \DateTimeZone('utc'))->format('c');
    $this->logger->notice(
      'Queueing all Orange DAM content for re-syncing with changes since :time.', [
        ':time' => $time,
      ]);
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Get the items deleted since the given time until now.
    $itemCount = $this->orangeDamApi->getContent($timeUpdatedSince, $timeNow);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the last time this was run for future checks.
    $this->state->set(static::UPDATED_ITEM_STATE_TIME_KEY, $timeNow->format('c'));
    // Log the results.
    $this->logger->notice(
      'Queued :count Orange DAM content item(s) for re-syncing with changes after :time.', [
        ':time' => $time,
        ':count' => $itemCount,
      ]);
    return $itemCount;
  }

  /**
   * {@inheritdoc}
   */
  public function queueUpdatedKeywords(DrupalDateTime $timeUpdatedSince = NULL) {
    // Use the passed time if available. Fallback to the updated time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeUpdatedSince)) {
      $timeUpdatedSince = new DrupalDateTime($this->state->get(static::UPDATED_KEYWORD_STATE_TIME_KEY, 'now'));
    }
    // Standardize the timezone.
    $timeUpdatedSince->setTimezone(new \DateTimeZone('utc'));
    // Record the time of the request.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Log the action being taken.
    $this->logger->notice(
      'Queueing all Orange DAM keywords for re-syncing that have been updated since :time.', [
        ':time' => $timeUpdatedSince->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $updatedKeywordCount = $this->orangeDamApi->getKeywords($timeUpdatedSince, $timeNow);
    if ($updatedKeywordCount === FALSE) {
      return FALSE;
    }
    // Record the last time this was run for future checks.
    $this->state->set(static::UPDATED_KEYWORD_STATE_TIME_KEY, $timeNow->format('c'));
    // Log the results.
    $this->logger->notice(
      'Queued :count Orange DAM keyword(s) for re-syncing.', [
        ':count' => $updatedKeywordCount,
      ]);
    return $updatedKeywordCount;
  }

  /**
   * Queue updates for all items of a given keyword type.
   *
   * @param string[] $keywordTypes
   *   The keyword types to queue updates for.
   *
   * @return int
   *   The number of queued keywords.
   */
  public function queueKeywordByType(array $keywordTypes): int {
    // Validate the keyword types.
    foreach ($keywordTypes as $keywordType) {
      if (!$this->validateKeywordType($keywordType)) {
        $this->logger->error('Invalid keyword type :type requested for syncing.', [
          ':type' => $keywordType,
        ]);
      }
    }
    // Request the data from the API and queue the results.
    $updatedKeywordCount = $this->orangeDamApi->getKeywords(
      NULL,
      NULL,
      $keywordTypes,
    );
    // Log the results.
    $this->logger->notice(
      'Queueing :count Orange DAM :type keywords(s) for re-syncing.', [
        ':count' => $updatedKeywordCount,
        ':type' => implode(', ', $keywordTypes),
      ]);
    return $updatedKeywordCount;
  }

  /**
   * {@inheritdoc}
   */
  public function updateSingleItem(string $systemIdentifier) {
    if ($item = $this->orangeDamApi->getContentItem($systemIdentifier)) {
      $this->logger->notice(
        'Orange DAM item :system_identifier has been successfully queued for updating in the :queue queue.', [
          ':system_identifier' => $systemIdentifier,
          ':queue' => $this->migrationDataManager->getItemQueueName($item),
        ]);
      return $item;
    }
    $this->logger->error(
      'Failed to retrieve Orange DAM item :system_id from the Orange DAM API.', [
        ':system_id' => $systemIdentifier,
      ]
    );
    return FALSE;
  }

  /**
   * Queue updates for all items of a given content type.
   *
   * @param array $contentTypes
   *   The content types to queue updates for.
   * @param int $limit
   *   The maximum number of items, rounded to the nearest page size.
   * @param int $pageNumber
   *   The optional initial page number for paged requests.
   * @param string $sort
   *   The optional Orang Logic API sort option.
   */
  public function queueContentByType(array $contentTypes, int $limit = 0, int $pageNumber = OrangeDamApi::INITIAL_PAGE_NUMBER, string $sort = '') {
    $itemCount = 0;
    // Validate the content types.
    foreach ($contentTypes as $contentType) {
      if (!$this->validateContentType($contentType)) {
        $this->logger->error(
          'Invalid content type :type requested for syncing.', [
            ':type' => $contentType,
          ]);
        continue;
      }
      // Request the data from the API and queue the results.
      $itemCount += $this->orangeDamApi->getContent(
        NULL,
        NULL,
        [$contentType],
        $limit,
        $pageNumber,
        $sort
      );
    }
    // Record the total number of items queued.
    $this->logger->notice(
      'Queued :count Orange DAM :types item(s) for re-syncing.', [
        ':count' => $itemCount,
        ':types' => implode(', ', $contentTypes),
      ]);
  }

  /**
   * {@inheritdoc}
   */
  public function queueContentDeletions(DrupalDateTime $timeDeletedSince = NULL) {
    // Use the passed time if available. Fallback to the deleted time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeDeletedSince)) {
      $timeDeletedSince = new DrupalDateTime($this->state->get(static::DELETED_CONTENT_STATE_TIME_KEY, 'now'));
    }
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));
    // Log the action being taken.
    $this->logger->notice(
      'Syncing deletions for all Orange DAM content that has been deleted since :time.', [
        ':time' => $timeDeletedSince->setTimezone(new \DateTimeZone('utc'))->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $itemCount = $this->orangeDamApi->getDeletedContent($timeDeletedSince, $timeNow);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the time since the request succeeded.
    $this->state->set(static::DELETED_CONTENT_STATE_TIME_KEY, $timeNow->format('c'));
    // Display a final summary.
    $this->logger->notice(
      'Queued deletions for :count Orange DAM content items.', [
        ':count' => $itemCount,
      ]);
    return $itemCount;
  }

  /**
   * {@inheritdoc}
   */
  public function queueKeywordDeletions(DrupalDateTime $timeDeletedSince = NULL) {
    // Use the passed time if available. Fallback to the deleted time state
    // value, which defaults to now if this is the first run.
    if (is_null($timeDeletedSince)) {
      $timeDeletedSince = new DrupalDateTime($this->state->get(static::DELETED_KEYWORD_STATE_TIME_KEY, 'now'));
    }
    // Record the last time this was run for future checks.
    $timeNow = new DrupalDateTime();
    $timeNow->setTimezone(new \DateTimeZone('utc'));

    $this->logger->notice(
      'Syncing deletions for all Orange DAM keywords that have been deleted since :time.', [
        ':time' => $timeDeletedSince->setTimezone(new \DateTimeZone('utc'))->format('c'),
      ]);
    // Get the items updated since the given time until now.
    $itemCount = $this->orangeDamApi->getDeletedKeywords($timeDeletedSince);
    if ($itemCount === FALSE) {
      return FALSE;
    }
    // Record the time since the request succeeded.
    $this->state->set(static::DELETED_KEYWORD_STATE_TIME_KEY, $timeNow->format('c'));
    return $itemCount;
  }

  /**
   * Deletes an Orange DAM item during queue processing.
   *
   * @param \Drupal\orange_dam\OrangeDamItemInterface $item
   *   The Orange DAM item to use as the source of the deletion operation.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *
   * @see \Drupal\orange_dam\Plugin\QueueWorker\OrangeDamDeletionQueueWorker
   */
  public function processDeletedQueueItem(OrangeDamItemInterface $item) {
    // Attempt to find the item in a migration.
    if (!$migration = $this->migrationDataManager->getMigrationBySourceId($item->id())) {
      $this->logger->error('Failed to delete an item of type ":type" while processing the deletion queue. Unable to find a migration with ":id" as a source identifier.', [
        ':id' => $item->id(),
        ':type' => $item->type(),
      ]);
      return;
    }
    // Get the destination entity ID and type values.
    $entityIds = $migration->getIdMap()->lookupDestinationIds([$item->id()]);
    // The result of the destination ID lookup above returns a nested array of
    // source keys and their 1 to many destination keys even when there are no
    // results, so we filter the results to simplify the empty check below.
    $entityIds = array_filter(array_pop($entityIds));
    $entityType = $migration->getDestinationPlugin()->getDerivativeId();
    // Validate that a destination entity was found.
    if ($entityIds === []) {
      $this->logger->error('Failed to delete an item of type ":type" while processing the deletion queue. Unable to find a migration destination item with ":id" as a source identifier.', [
        ':id' => $item->id(),
        ':type' => $item->type(),
      ]);
      return;
    }
    // Load the entities.
    $entities = $this->entityTypeManager
      ->getStorage($entityType)
      ->loadMultiple($entityIds);
    // Validate that entities were found.
    if ($entities === []) {
      $this->logger->error('Failed to find an entity with the ID :id of the Orange Logic type ":type" while processing deletions for item ":source_id".', [
        ':id' => implode(', ', $entityIds),
        ':type' => $item->type(),
        ':source_id' => $item->id(),
      ]);
      return;
    }
    // Delete the entities.
    foreach ($entities as $entity) {
      try {
        $entity->delete();
        $this->logger->info('Deleted entity (:type::id) ":title" during Orange DAM deletion syncing.', [
          ':type' => $entity->getEntityType()->id(),
          ':id' => $entity->id(),
          ':title' => $entity->label(),
        ]);
      }
      catch (EntityStorageException) {
        $this->logger->error('Failed to delete entity (:type::id) ":title" during Orange DAM deletion syncing.', [
          ':type' => $entity->getEntityType(),
          ':id' => $entity->id(),
          ':title' => $entity->label(),
        ]);
        continue;
      }
    }
  }

  /**
   * Validate if a content type is valid.
   *
   * @param string $contentType
   *   The content type name.
   *
   * @return bool
   *   Indication if the content type is valid.
   */
  public function validateContentType($contentType): bool {
    return in_array($contentType, $this->orangeDamConfiguration->getContentTypeNames());
  }

  /**
   * Validate if a keyword type is valid.
   *
   * @param string $keywordType
   *   The keyword type name.
   *
   * @return bool
   *   Indication if the keyword type is valid.
   */
  public function validateKeywordType($keywordType): bool {
    return in_array($keywordType, $this->orangeDamConfiguration->getKeywordTypes());
  }

  /**
   * Generate the URL to a file asset for a given item.
   *
   * The content type is optional, but if it is omitted it adds a second API
   * call to this process.
   *
   * @param string $systemId
   *   The item's System ID.
   * @param string $contentType
   *   The item's content type.
   *
   * @return string|false
   *   The asset URL if one was generated.
   */
  public function getAssetUrl(string $systemId, string $contentType = NULL) {
    // Get the content type of item if it was not passed in.
    if (is_null($contentType)) {
      $item = $this->orangeDamApi->getContentItem($systemId);
      $contentType = $item ? $item->type() : NULL;
    }
    // Confirm that a content type was found.
    if (!$contentType) {
      $this->logger->error(
        'Unable to determine the content type of the item with system ID :system_id for asset URL generation.', [
          ':system_id' => $systemId,
        ]);
      return FALSE;
    }
    // Get the asset format and verify that a value was found.
    $assetFormat = $this->orangeDamConfiguration->getAssetFormat($contentType);
    if (!$assetFormat) {
      return FALSE;
    }
    // Generate the asset URL.
    return $this->orangeDamApi->getAssetUrl(
      $systemId,
      $assetFormat,
    );
  }

}
